<?php

namespace chillerlan\FastRoute\Dispatcher;

/**
 * @method dispatchVariableRoute(array $routeData, string $uri):array
 */
abstract class DispatcherAbstract implements DispatcherInterface{

	protected $staticRouteMap    = [];
	protected $variableRouteData = [];

	public function __construct(array $data){
		$this->staticRouteMap    = $data[0];
		$this->variableRouteData = $data[1];
	}

	public function dispatch(string $httpMethod, string $uri):array{

		if(isset($this->staticRouteMap[$httpMethod][$uri])){
			return [
				self::FOUND,
				$this->staticRouteMap[$httpMethod][$uri],
				[],
			];
		}
		elseif($httpMethod === 'HEAD' && isset($this->staticRouteMap['GET'][$uri])){
			return [
				self::FOUND,
				$this->staticRouteMap['GET'][$uri],
				[],
			];
		}

		if(isset($this->variableRouteData[$httpMethod])){
			$result = $this->dispatchVariableRoute($this->variableRouteData[$httpMethod], $uri);
			if($result[0] === self::FOUND){
				return $result;
			}
		}
		elseif($httpMethod === 'HEAD' && isset($this->variableRouteData['GET'])){
			$result = $this->dispatchVariableRoute($this->variableRouteData['GET'], $uri);
			if($result[0] === self::FOUND){
				return $result;
			}
		}

		// Find allowed methods for this URI by matching against all other HTTP methods as well
		$allowedMethods = [];
		foreach($this->staticRouteMap as $method => $uriMap){
			if($method !== $httpMethod && isset($uriMap[$uri])){
				$allowedMethods[] = $method;
			}
		}

		foreach($this->variableRouteData as $method => $routeData){
			if($method === $httpMethod){
				continue;
			}

			if($this->dispatchVariableRoute($routeData, $uri)[0] === self::FOUND){
				$allowedMethods[] = $method;
			}
		}

		// If there are no allowed methods the route simply does not exist
		return $allowedMethods
			? [self::METHOD_NOT_ALLOWED, $allowedMethods]
			: [self::NOT_FOUND];
	}

}
