<?php

namespace chillerlan\FastRoute;

use chillerlan\FastRoute\DataGenerator\GeneratorCCB;
use chillerlan\FastRoute\DataGenerator\GeneratorGCB;
use chillerlan\FastRoute\DataGenerator\GeneratorGPB;
use chillerlan\FastRoute\DataGenerator\GeneratorMB;
use chillerlan\FastRoute\Dispatcher\DispatcherInterface;
use chillerlan\FastRoute\Dispatcher\DispatcherCCB;
use chillerlan\FastRoute\Dispatcher\DispatcherGCB;
use chillerlan\FastRoute\Dispatcher\DispatcherGPB;
use chillerlan\FastRoute\Dispatcher\DispatcherMB;
use chillerlan\FastRoute\RouteParser\RouteParserStd;

class Router{

	const CCB = [GeneratorCCB::class, DispatcherCCB::class];
	const GCB = [GeneratorGCB::class, DispatcherGCB::class];
	const GPB = [GeneratorGPB::class, DispatcherGPB::class];
	const MB  = [GeneratorMB::class,  DispatcherMB::class ];

	public function getDispatcher(callable $callback, array $mode = self::GCB):DispatcherInterface{
		list($generator, $dispatcher) = $mode;
		
		$routeCollector = new RouteCollector(new RouteParserStd, new $generator);
		$callback($routeCollector);

		return new $dispatcher($routeCollector->getData());
	}

	/**
	 * @param callable $routeDefinitionCallback
	 * @param array    $mode
	 * @param array    $options ['cacheFile' => string, 'cacheDisabled' => bool]
	 *
	 * @return \chillerlan\FastRoute\Dispatcher\DispatcherInterface
	 *
	 * @codeCoverageIgnore
	 */
	public function getCachedDispatcher(callable $routeDefinitionCallback, array $mode = self::GCB, array $options = []):DispatcherInterface{
		list($generator, $dispatcher) = $mode;

		$options += [
			'cacheDisabled'  => false,
		];

		if(!isset($options['cacheFile'])){
			throw new \LogicException('Must specify "cacheFile" option');
		}

		if(!$options['cacheDisabled'] && file_exists($options['cacheFile'])){
			$dispatchData = require $options['cacheFile'];

			if(!is_array($dispatchData)){
				throw new \RuntimeException('Invalid cache file "'.$options['cacheFile'].'"');
			}

		}
		else{
			$routeCollector = new RouteCollector(new RouteParserStd, new $generator);
			$routeDefinitionCallback($routeCollector);

			$dispatchData = $routeCollector->getData();
			file_put_contents($options['cacheFile'], '<?php return '.var_export($dispatchData, true).';');
		}

		return new $dispatcher($dispatchData);
	}

}
